/*
 *  jDTAUS - DTAUS fileformat.
 *  Copyright (c) 2005 Christian Schulte <cs@schulte.it>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */
package org.jdtaus.core.messages;

import java.util.Locale;
import org.jdtaus.core.text.Message;

/**
 * {@code Message} stating that an exception occured.
 * <p>The {@code Throwable} passed to the constructor of this class is used to
 * determine the message's format arguments. For chained exceptions, that is,
 * {@code getCause()} returns a non-{@code null} value, the root cause is used.
 * </p>
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: ExceptionMessage.java 4146 2007-07-30 00:01:20Z schulte2005 $
 */
public final class ExceptionMessage extends Message
{
    //--ExceptionMessage--------------------------------------------------------

    /**
     * Throwable of the message.
     * @serial
     */
    private final Throwable throwable;

    /**
     * Creates a new {@code ExceptionMessage} taking the {@code Throwable} to
     * report.
     *
     * @param throwable the {@code Throwable} to report.
     *
     * @throws NullPointerException if {@code throwable} is {@code null}.
     */
    public ExceptionMessage(final Throwable throwable)
    {
        if(throwable == null)
        {
            throw new NullPointerException("throwable");
        }

        this.throwable = throwable;
    }

    //--------------------------------------------------------ExceptionMessage--
    //--Message-----------------------------------------------------------------

    /**
     * {@inheritDoc}
     * <p>This method traverses up the chained hierarchy. The arguments
     * returned are constructed using the root cause.</p>
     *
     * @return Strings describing the throwable.
     * <ul>
     * <li>[0]: the fully qualified classname of the root cause.</li>
     * <li>[1]: the classname of the root cause without package.</li>
     * <li>[2]: the message of the root cause.</li>
     * </ul>
     */
    public Object[] getFormatArguments(final Locale locale)
    {
        Throwable current = this.throwable;
        Throwable report = current;

        while((current = current.getCause()) != null)
        {
            report = current;
        }

        String name = report.getClass().getName();
        final int lastDot = name.lastIndexOf('.');

        if(lastDot >= 0)
        {
            name = name.substring(lastDot + 1);
        }

        String message = report.getMessage();
        if(message == null)
        {
            message = ExceptionMessageBundle.getNullMessageText(
                Locale.getDefault());

        }

        return new Object[] {
            report.getClass().getName(), name, message
        };
    }

    /**
     * {@inheritDoc}
     *
     * @return The corresponding text from the message's {@code ResourceBundle}
     * (defaults to "An {1} occured.\n{2}").
     */
    public String getText(final Locale locale)
    {
        return ExceptionMessageBundle.getThrowableInfoMessage(locale).
            format(this.getFormatArguments(locale));

    }

    //-----------------------------------------------------------------Message--
}
